# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/vpcs/ui/vpcs_node_preferences_page.ui'
#
# Created: Sun Jul 10 16:48:58 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_VPCSNodePageWidget(object):
    def setupUi(self, VPCSNodePageWidget):
        VPCSNodePageWidget.setObjectName("VPCSNodePageWidget")
        VPCSNodePageWidget.resize(546, 455)
        VPCSNodePageWidget.setAccessibleDescription("")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(VPCSNodePageWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter = QtWidgets.QSplitter(VPCSNodePageWidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.uiVPCSTreeWidget = QtWidgets.QTreeWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVPCSTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiVPCSTreeWidget.setSizePolicy(sizePolicy)
        self.uiVPCSTreeWidget.setMaximumSize(QtCore.QSize(160, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.uiVPCSTreeWidget.setFont(font)
        self.uiVPCSTreeWidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.uiVPCSTreeWidget.setIconSize(QtCore.QSize(32, 32))
        self.uiVPCSTreeWidget.setRootIsDecorated(False)
        self.uiVPCSTreeWidget.setObjectName("uiVPCSTreeWidget")
        self.uiVPCSTreeWidget.headerItem().setText(0, "1")
        self.uiVPCSTreeWidget.header().setVisible(False)
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiVPCSInfoTreeWidget = QtWidgets.QTreeWidget(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVPCSInfoTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiVPCSInfoTreeWidget.setSizePolicy(sizePolicy)
        self.uiVPCSInfoTreeWidget.setIndentation(10)
        self.uiVPCSInfoTreeWidget.setAllColumnsShowFocus(True)
        self.uiVPCSInfoTreeWidget.setObjectName("uiVPCSInfoTreeWidget")
        self.uiVPCSInfoTreeWidget.header().setVisible(False)
        self.verticalLayout.addWidget(self.uiVPCSInfoTreeWidget)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiNewVPCSPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiNewVPCSPushButton.setObjectName("uiNewVPCSPushButton")
        self.horizontalLayout_5.addWidget(self.uiNewVPCSPushButton)
        self.uiEditVPCSPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiEditVPCSPushButton.setEnabled(False)
        self.uiEditVPCSPushButton.setObjectName("uiEditVPCSPushButton")
        self.horizontalLayout_5.addWidget(self.uiEditVPCSPushButton)
        self.uiDeleteVPCSPushButton = QtWidgets.QPushButton(self.layoutWidget)
        self.uiDeleteVPCSPushButton.setEnabled(False)
        self.uiDeleteVPCSPushButton.setObjectName("uiDeleteVPCSPushButton")
        self.horizontalLayout_5.addWidget(self.uiDeleteVPCSPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.verticalLayout_2.addWidget(self.splitter)

        self.retranslateUi(VPCSNodePageWidget)
        QtCore.QMetaObject.connectSlotsByName(VPCSNodePageWidget)
        VPCSNodePageWidget.setTabOrder(self.uiNewVPCSPushButton, self.uiDeleteVPCSPushButton)

    def retranslateUi(self, VPCSNodePageWidget):
        _translate = QtCore.QCoreApplication.translate
        VPCSNodePageWidget.setWindowTitle(_translate("VPCSNodePageWidget", "VPCS nodes"))
        VPCSNodePageWidget.setAccessibleName(_translate("VPCSNodePageWidget", "VPCS node templates"))
        self.uiVPCSInfoTreeWidget.headerItem().setText(0, _translate("VPCSNodePageWidget", "1"))
        self.uiVPCSInfoTreeWidget.headerItem().setText(1, _translate("VPCSNodePageWidget", "2"))
        self.uiNewVPCSPushButton.setText(_translate("VPCSNodePageWidget", "&New"))
        self.uiEditVPCSPushButton.setText(_translate("VPCSNodePageWidget", "&Edit"))
        self.uiDeleteVPCSPushButton.setText(_translate("VPCSNodePageWidget", "&Delete"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
IOU module implementation.
"""

import sys
import os
import shutil

from gns3.qt import QtWidgets
from gns3.local_config import LocalConfig

from ..module import Module
from ..module_error import ModuleError
from .iou_device import IOUDevice
from .settings import IOU_SETTINGS
from .settings import IOU_DEVICE_SETTINGS

import logging
log = logging.getLogger(__name__)


class IOU(Module):

    """
    IOU module.
    """

    def __init__(self):
        super().__init__()

        self._settings = {}
        self._nodes = []
        self._iou_devices = {}
        self._iou_images_cache = {}

        self.configChangedSlot()

    def configChangedSlot(self):
        # load the settings
        self._loadSettings()

    def _loadSettings(self):
        """
        Loads the settings from the persistent settings file.
        """

        self._settings = LocalConfig.instance().loadSectionSettings(self.__class__.__name__, IOU_SETTINGS)
        self._loadIOUDevices()

    def _saveSettings(self):
        """
        Saves the settings to the persistent settings file.
        """

        # save the settings
        LocalConfig.instance().saveSectionSettings(self.__class__.__name__, self._settings)

    def _loadIOUDevices(self):
        """
        Load the IOU devices from the persistent settings file.
        """

        self._iou_devices = {}
        settings = LocalConfig.instance().settings()
        if "devices" in settings.get(self.__class__.__name__, {}):
            for device in settings[self.__class__.__name__]["devices"]:
                name = device.get("name")
                server = device.get("server")
                key = "{server}:{name}".format(server=server, name=name)
                if key in self._iou_devices or not name or not server:
                    continue
                device_settings = IOU_DEVICE_SETTINGS.copy()
                device_settings.update(device)
                # for backward compatibility before version 1.4
                if "symbol" not in device_settings:
                    device_settings["symbol"] = device_settings["default_symbol"]
                    device_settings["symbol"] = device_settings["symbol"][:-11] + ".svg" if device_settings["symbol"].endswith("normal.svg") else device_settings["symbol"]
                device_settings["startup_config"] = device_settings.get("initial_config", device_settings["startup_config"])
                self._iou_devices[key] = device_settings

    def _saveIOUDevices(self):
        """
        Saves the IOU devices to the persistent settings file.
        """

        self._settings["devices"] = list(self._iou_devices.values())
        self._saveSettings()

    def addNode(self, node):
        """
        Adds a node to this module.

        :param node: Node instance
        """

        self._nodes.append(node)

    def removeNode(self, node):
        """
        Removes a node from this module.

        :param node: Node instance
        """

        if node in self._nodes:
            self._nodes.remove(node)

    def VMs(self):
        """
        Returns IOU devices settings.

        :returns: IOU devices settings (dictionary)
        """

        return self._iou_devices

    def setVMs(self, new_iou_devices):
        """
        Sets IOS devices settings.

        :param new_iou_devices: IOU images settings (dictionary)
        """

        self._iou_devices = new_iou_devices.copy()
        self._saveIOUDevices()

    @staticmethod
    def vmConfigurationPage():
        from .pages.iou_device_configuration_page import iouDeviceConfigurationPage
        return iouDeviceConfigurationPage

    def settings(self):
        """
        Returns the module settings

        :returns: module settings (dictionary)
        """

        return self._settings

    def setSettings(self, settings):
        """
        Sets the module settings

        :param settings: module settings (dictionary)
        """

        self._settings.update(settings)
        self._saveSettings()

    def instantiateNode(self, node_class, server, project):
        """
        Instantiate a new node.

        :param node_class: Node object
        :param server: HTTPClient instance
        :param project: Project instance
        """

        # create an instance of the node class
        return node_class(self, server, project)

    def reset(self):
        """
        Resets the servers.
        """

        self._nodes.clear()

    def findAlternativeIOUImage(self, image):
        """
        Tries to find an alternative IOU image

        :param image: path to IOU

        :return: IOU image path
        """

        if image in self._iou_images_cache:
            return self._iou_images_cache[image]

        from gns3.main_window import MainWindow
        mainwindow = MainWindow.instance()
        iou_devices = self.VMs()
        candidate_iou_images = {}

        alternative_image = image

        # find all images with the same platform and local server
        for iou_device in iou_devices.values():
            if iou_device["server"] == "local":
                candidate_iou_images[iou_device["image"]] = iou_device["path"]

        if candidate_iou_images:
            selection, ok = QtWidgets.QInputDialog.getItem(mainwindow,
                                                           "IOU image", "IOU image {} could not be found\nPlease select an alternative from your existing images:".format(image),
                                                           list(candidate_iou_images.keys()), 0, False)
            if ok:
                iou_image = candidate_iou_images[selection]
                self._iou_images_cache[image] = iou_image
                return iou_image

        # no registered IOU image is used, let's just ask for an IOU image path
        QtWidgets.QMessageBox.critical(mainwindow, "IOU image", "Could not find the {} IOU image \nPlease select a similar IOU image!".format(image))
        from .pages.iou_device_preferences_page import IOUDevicePreferencesPage
        path = IOUDevicePreferencesPage.getIOUImage(mainwindow, None)
        if path:
            alternative_image = path
            self._iou_images_cache[image] = alternative_image
        return alternative_image

    @staticmethod
    def getNodeClass(name):
        """
        Returns the object with the corresponding name.

        :param name: object name
        """

        if name in globals():
            return globals()[name]
        return None

    @staticmethod
    def getNodeType(name, platform=None):
        if name == "iou":
            return IOUDevice
        return None

    @staticmethod
    def classes():
        """
        Returns all the node classes supported by this module.

        :returns: list of classes
        """

        return [IOUDevice]

    def nodes(self):
        """
        Returns all the node data necessary to represent a node
        in the nodes view and create a node on the scene.
        """

        nodes = []
        for iou_device in self._iou_devices.values():
            nodes.append(
                {"class": IOUDevice.__name__,
                 "name": iou_device["name"],
                 "server": iou_device["server"],
                 "symbol": iou_device["symbol"],
                 "categories": [iou_device["category"]]
                 }
            )
        return nodes

    @staticmethod
    def preferencePages():
        """
        :returns: QWidget object list
        """

        from .pages.iou_preferences_page import IOUPreferencesPage
        from .pages.iou_device_preferences_page import IOUDevicePreferencesPage
        return [IOUPreferencesPage, IOUDevicePreferencesPage]

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of IOU module.

        :returns: instance of IOU
        """

        if not hasattr(IOU, "_instance"):
            IOU._instance = IOU()
        return IOU._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for TraceNG preferences.
"""

import os
import sys
import shutil

from gns3.qt import QtWidgets

from .. import TraceNG
from ..ui.traceng_preferences_page_ui import Ui_TraceNGPreferencesPageWidget
from ..settings import TRACENG_SETTINGS


class TraceNGPreferencesPage(QtWidgets.QWidget, Ui_TraceNGPreferencesPageWidget):

    """
    QWidget preference page for TraceNG
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)

        # connect signals
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)
        self.uiTraceNGPathToolButton.clicked.connect(self._tracengPathBrowserSlot)

    def _tracengPathBrowserSlot(self):
        """
        Slot to open a file browser and select traceng
        """

        filter = ""
        if sys.platform.startswith("win"):
            filter = "Executable (*.exe);;All files (*.*)"
        traceng_path = shutil.which("traceng")
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select TraceNG", traceng_path, filter)
        if not path:
            return

        if self._checkTraceNGPath(path):
            self.uiTraceNGPathLineEdit.setText(os.path.normpath(path))

    def _checkTraceNGPath(self, path):
        """
        Checks that the TraceNG path is valid.

        :param path: TraceNG path
        :returns: boolean
        """

        if not os.path.exists(path):
            QtWidgets.QMessageBox.critical(self, "TraceNG", '"{}" does not exist'.format(path))
            return False

        if not os.access(path, os.X_OK):
            QtWidgets.QMessageBox.critical(self, "TraceNG", "{} is not an executable".format(os.path.basename(path)))
            return False

        return True

    def _restoreDefaultsSlot(self):
        """
        Slot to populate the page widgets with the default settings.
        """

        self._populateWidgets(TRACENG_SETTINGS)

    def _useLocalServerSlot(self, state):
        """
        Slot to enable or not local server settings.
        """

        if state:
            self.uiTraceNGPathLineEdit.setEnabled(True)
            self.uiTraceNGPathToolButton.setEnabled(True)
        else:
            self.uiTraceNGPathLineEdit.setEnabled(False)
            self.uiTraceNGPathToolButton.setEnabled(False)

    def _populateWidgets(self, settings):
        """
        Populates the widgets with the settings.

        :param settings: TraceNG settings
        """

        self.uiTraceNGPathLineEdit.setText(settings["traceng_path"])

    def loadPreferences(self):
        """
        Loads TraceNG preferences.
        """

        traceng_settings = TraceNG.instance().settings()
        self._populateWidgets(traceng_settings)

    def savePreferences(self):
        """
        Saves TraceNG preferences.
        """

        traceng_path = self.uiTraceNGPathLineEdit.text().strip()
        if traceng_path and not self._checkTraceNGPath(traceng_path):
            return
        new_settings = {"traceng_path": traceng_path}
        TraceNG.instance().setSettings(new_settings)



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .qt import QtWidgets
from .settings import NODES_VIEW_SETTINGS
from .local_config import LocalConfig


class NodesDockWidget(QtWidgets.QDockWidget):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._settings = LocalConfig.instance().loadSectionSettings("NodesView", NODES_VIEW_SETTINGS)

    def _filterTextChangedSlot(self, text):
        self.parent().uiNodesView.setCurrentSearch(text)
        self.parent().uiNodesView.refresh()

    def _filterIndexChangedSlot(self, index):
        self._settings["nodes_view_filter"] = index
        LocalConfig.instance().saveSectionSettings("NodesView", self._settings)

        if index == 0:
            self.parent().uiNodesView.setShowInstalledAppliances(True)
            self.parent().uiNodesView.setShowBuiltinAvailableAppliances(True)
            self.parent().uiNodesView.setShowMyAvailableAppliances(True)
        elif index == 1:
            self.parent().uiNodesView.setShowInstalledAppliances(True)
            self.parent().uiNodesView.setShowBuiltinAvailableAppliances(False)
            self.parent().uiNodesView.setShowMyAvailableAppliances(False)
        elif index == 2:
            self.parent().uiNodesView.setShowInstalledAppliances(False)
            self.parent().uiNodesView.setShowBuiltinAvailableAppliances(True)
            self.parent().uiNodesView.setShowMyAvailableAppliances(True)
        else:
            self.parent().uiNodesView.setShowInstalledAppliances(False)
            self.parent().uiNodesView.setShowBuiltinAvailableAppliances(False)
            self.parent().uiNodesView.setShowMyAvailableAppliances(True)
        self.parent().uiNodesView.refresh()

    def populateNodesView(self, category):
        if self.parent().uiNodesFilterComboBox.currentIndex() != self._settings["nodes_view_filter"]:
            self.parent().uiNodesFilterComboBox.setCurrentIndex(self._settings["nodes_view_filter"])
            self._filterIndexChangedSlot(self._settings["nodes_view_filter"])
        self.parent().uiNodesFilterComboBox.activated.connect(self._filterIndexChangedSlot)
        self.parent().uiNodesFilterLineEdit.textChanged.connect(self._filterTextChangedSlot)
        self.parent().uiNodesView.clear()
        text = self.parent().uiNodesFilterLineEdit.text().strip().lower()
        self.parent().uiNodesView.populateNodesView(category, text)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/docker/ui/docker_vm_configuration_page.ui'
#
# Created: Thu Jan  5 14:49:45 2017
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_dockerVMConfigPageWidget(object):
    def setupUi(self, dockerVMConfigPageWidget):
        dockerVMConfigPageWidget.setObjectName("dockerVMConfigPageWidget")
        dockerVMConfigPageWidget.resize(613, 519)
        self.verticalLayout = QtWidgets.QVBoxLayout(dockerVMConfigPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiTabWidget = QtWidgets.QTabWidget(dockerVMConfigPageWidget)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout = QtWidgets.QGridLayout(self.tab)
        self.gridLayout.setContentsMargins(10, 10, 10, 10)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.tab)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiConsoleTypeLabel = QtWidgets.QLabel(self.tab)
        self.uiConsoleTypeLabel.setObjectName("uiConsoleTypeLabel")
        self.gridLayout.addWidget(self.uiConsoleTypeLabel, 6, 0, 1, 1)
        self.uiAdapterLabel = QtWidgets.QLabel(self.tab)
        self.uiAdapterLabel.setObjectName("uiAdapterLabel")
        self.gridLayout.addWidget(self.uiAdapterLabel, 5, 0, 1, 1)
        self.uiCMDLineEdit = QtWidgets.QLineEdit(self.tab)
        self.uiCMDLineEdit.setObjectName("uiCMDLineEdit")
        self.gridLayout.addWidget(self.uiCMDLineEdit, 4, 1, 1, 1)
        self.label = QtWidgets.QLabel(self.tab)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 8, 0, 1, 1)
        self.uiConsoleHttpPortSpinBox = QtWidgets.QSpinBox(self.tab)
        self.uiConsoleHttpPortSpinBox.setMinimum(1)
        self.uiConsoleHttpPortSpinBox.setMaximum(65535)
        self.uiConsoleHttpPortSpinBox.setObjectName("uiConsoleHttpPortSpinBox")
        self.gridLayout.addWidget(self.uiConsoleHttpPortSpinBox, 8, 1, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(self.tab)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.horizontalLayout_7.addWidget(self.uiSymbolLineEdit)
        self.uiSymbolToolButton = QtWidgets.QToolButton(self.tab)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.horizontalLayout_7.addWidget(self.uiSymbolToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_7, 3, 1, 1, 1)
        self.uiDefaultNameFormatLabel = QtWidgets.QLabel(self.tab)
        self.uiDefaultNameFormatLabel.setObjectName("uiDefaultNameFormatLabel")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLabel, 1, 0, 1, 1)
        self.uiEnvironmentLabel = QtWidgets.QLabel(self.tab)
        self.uiEnvironmentLabel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.uiEnvironmentLabel.setWordWrap(False)
        self.uiEnvironmentLabel.setObjectName("uiEnvironmentLabel")
        self.gridLayout.addWidget(self.uiEnvironmentLabel, 10, 0, 1, 1)
        self.uiNetworkConfigEditButton = QtWidgets.QPushButton(self.tab)
        self.uiNetworkConfigEditButton.setObjectName("uiNetworkConfigEditButton")
        self.gridLayout.addWidget(self.uiNetworkConfigEditButton, 11, 1, 1, 1)
        self.uiConsoleTypeComboBox = QtWidgets.QComboBox(self.tab)
        self.uiConsoleTypeComboBox.setObjectName("uiConsoleTypeComboBox")
        self.uiConsoleTypeComboBox.addItem("")
        self.uiConsoleTypeComboBox.addItem("")
        self.uiConsoleTypeComboBox.addItem("")
        self.uiConsoleTypeComboBox.addItem("")
        self.gridLayout.addWidget(self.uiConsoleTypeComboBox, 6, 1, 1, 1)
        self.uiNetworkConfigLabel = QtWidgets.QLabel(self.tab)
        self.uiNetworkConfigLabel.setObjectName("uiNetworkConfigLabel")
        self.gridLayout.addWidget(self.uiNetworkConfigLabel, 11, 0, 1, 1)
        self.uiDefaultNameFormatLineEdit = QtWidgets.QLineEdit(self.tab)
        self.uiDefaultNameFormatLineEdit.setObjectName("uiDefaultNameFormatLineEdit")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLineEdit, 1, 1, 1, 1)
        self.uiNameLabel = QtWidgets.QLabel(self.tab)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiAdapterSpinBox = QtWidgets.QSpinBox(self.tab)
        self.uiAdapterSpinBox.setMinimum(1)
        self.uiAdapterSpinBox.setObjectName("uiAdapterSpinBox")
        self.gridLayout.addWidget(self.uiAdapterSpinBox, 5, 1, 1, 1)
        self.uiEnvironmentTextEdit = QtWidgets.QTextEdit(self.tab)
        self.uiEnvironmentTextEdit.setObjectName("uiEnvironmentTextEdit")
        self.gridLayout.addWidget(self.uiEnvironmentTextEdit, 10, 1, 1, 1)
        self.uiCategoryComboBox = QtWidgets.QComboBox(self.tab)
        self.uiCategoryComboBox.setObjectName("uiCategoryComboBox")
        self.uiCategoryComboBox.addItem("")
        self.uiCategoryComboBox.addItem("")
        self.uiCategoryComboBox.addItem("")
        self.uiCategoryComboBox.addItem("")
        self.gridLayout.addWidget(self.uiCategoryComboBox, 2, 1, 1, 1)
        self.uiCategoryLabel = QtWidgets.QLabel(self.tab)
        self.uiCategoryLabel.setObjectName("uiCategoryLabel")
        self.gridLayout.addWidget(self.uiCategoryLabel, 2, 0, 1, 1)
        self.uiCMDLabel = QtWidgets.QLabel(self.tab)
        self.uiCMDLabel.setObjectName("uiCMDLabel")
        self.gridLayout.addWidget(self.uiCMDLabel, 4, 0, 1, 1)
        self.uiConsoleResolutionComboBox = QtWidgets.QComboBox(self.tab)
        self.uiConsoleResolutionComboBox.setObjectName("uiConsoleResolutionComboBox")
        self.uiConsoleResolutionComboBox.addItem("")
        self.uiConsoleResolutionComboBox.addItem("")
        self.uiConsoleResolutionComboBox.addItem("")
        self.uiConsoleResolutionComboBox.addItem("")
        self.uiConsoleResolutionComboBox.addItem("")
        self.uiConsoleResolutionComboBox.addItem("")
        self.uiConsoleResolutionComboBox.addItem("")
        self.gridLayout.addWidget(self.uiConsoleResolutionComboBox, 7, 1, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(self.tab)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout.addWidget(self.uiSymbolLabel, 3, 0, 1, 1)
        self.uiConsoleResolutionLabel = QtWidgets.QLabel(self.tab)
        self.uiConsoleResolutionLabel.setObjectName("uiConsoleResolutionLabel")
        self.gridLayout.addWidget(self.uiConsoleResolutionLabel, 7, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.tab)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 9, 0, 1, 1)
        self.uiHttpConsolePathLineEdit = QtWidgets.QLineEdit(self.tab)
        self.uiHttpConsolePathLineEdit.setObjectName("uiHttpConsolePathLineEdit")
        self.gridLayout.addWidget(self.uiHttpConsolePathLineEdit, 9, 1, 1, 1)
        self.uiTabWidget.addTab(self.tab, "")
        self.verticalLayout.addWidget(self.uiTabWidget)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(dockerVMConfigPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(dockerVMConfigPageWidget)

    def retranslateUi(self, dockerVMConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        dockerVMConfigPageWidget.setWindowTitle(_translate("dockerVMConfigPageWidget", "Docker image configuration"))
        self.uiConsoleTypeLabel.setText(_translate("dockerVMConfigPageWidget", "Console type:"))
        self.uiAdapterLabel.setText(_translate("dockerVMConfigPageWidget", "Adapters:"))
        self.label.setText(_translate("dockerVMConfigPageWidget", "HTTP port in the container:"))
        self.uiSymbolToolButton.setText(_translate("dockerVMConfigPageWidget", "&Browse..."))
        self.uiDefaultNameFormatLabel.setText(_translate("dockerVMConfigPageWidget", "Default name format"))
        self.uiEnvironmentLabel.setText(_translate("dockerVMConfigPageWidget", "Environment variables:\n"
"(KEY=VALUE, one per line)"))
        self.uiNetworkConfigEditButton.setText(_translate("dockerVMConfigPageWidget", "Edit"))
        self.uiConsoleTypeComboBox.setItemText(0, _translate("dockerVMConfigPageWidget", "telnet"))
        self.uiConsoleTypeComboBox.setItemText(1, _translate("dockerVMConfigPageWidget", "vnc"))
        self.uiConsoleTypeComboBox.setItemText(2, _translate("dockerVMConfigPageWidget", "http"))
        self.uiConsoleTypeComboBox.setItemText(3, _translate("dockerVMConfigPageWidget", "https"))
        self.uiNetworkConfigLabel.setText(_translate("dockerVMConfigPageWidget", "Network configuration"))
        self.uiNameLabel.setText(_translate("dockerVMConfigPageWidget", "Name:"))
        self.uiCategoryComboBox.setItemText(0, _translate("dockerVMConfigPageWidget", "Router"))
        self.uiCategoryComboBox.setItemText(1, _translate("dockerVMConfigPageWidget", "Switch"))
        self.uiCategoryComboBox.setItemText(2, _translate("dockerVMConfigPageWidget", "Guest"))
        self.uiCategoryComboBox.setItemText(3, _translate("dockerVMConfigPageWidget", "Security device"))
        self.uiCategoryLabel.setText(_translate("dockerVMConfigPageWidget", "Category"))
        self.uiCMDLabel.setText(_translate("dockerVMConfigPageWidget", "Start command:"))
        self.uiConsoleResolutionComboBox.setItemText(0, _translate("dockerVMConfigPageWidget", "1920x1080"))
        self.uiConsoleResolutionComboBox.setItemText(1, _translate("dockerVMConfigPageWidget", "1366x768"))
        self.uiConsoleResolutionComboBox.setItemText(2, _translate("dockerVMConfigPageWidget", "1280x1024"))
        self.uiConsoleResolutionComboBox.setItemText(3, _translate("dockerVMConfigPageWidget", "1280x800"))
        self.uiConsoleResolutionComboBox.setItemText(4, _translate("dockerVMConfigPageWidget", "1024x768"))
        self.uiConsoleResolutionComboBox.setItemText(5, _translate("dockerVMConfigPageWidget", "800x600"))
        self.uiConsoleResolutionComboBox.setItemText(6, _translate("dockerVMConfigPageWidget", "640x480"))
        self.uiSymbolLabel.setText(_translate("dockerVMConfigPageWidget", "Symbol:"))
        self.uiConsoleResolutionLabel.setText(_translate("dockerVMConfigPageWidget", "VNC console resolution:"))
        self.label_2.setText(_translate("dockerVMConfigPageWidget", "HTTP path:"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.tab), _translate("dockerVMConfigPageWidget", "General settings"))




#!/usr/bin/env python
#
# Copyright (C) 2017 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..qt import QtCore
from ..controller import Controller

import logging
log = logging.getLogger(__name__)


class CreateSnapshotWorker(QtCore.QObject):
    """
    Export snapshot
    """

    # signals to update the progress dialog.
    error = QtCore.pyqtSignal(str, bool)
    finished = QtCore.pyqtSignal()
    updated = QtCore.pyqtSignal(int)

    canceled = QtCore.pyqtSignal()


    def __init__(self, project, snapshot_name):
        super().__init__()
        self._project = project
        self._snapshot_name = snapshot_name

    def run(self):
        if self._project:
            Controller.instance().post(
                "/projects/{}/snapshots".format(self._project.id()),
                self._createSnapshotsCallback,
                {"name": self._snapshot_name},
                timeout=None,
                showProgress=False,
                eventsHandler=self
            )

    def _createSnapshotsCallback(self, content, error=False, server=None, context={}, **kwargs):
        if error:
            if content:
                self.error.emit(content["message"], True)
            else:
                self.error.emit("Cannot create snapshot of project", True)
            self.finished.emit()
            return
        self.finished.emit()

    def cancel(self):
        log.warning("Snapshot `{}` creation has been canceled by user.".format(self._snapshot_name))
        self.canceled.emit()



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sip
import shutil

from ..qt import QtWidgets, QtCore, QtGui, qpartial, qslot
from ..ui.appliance_wizard_ui import Ui_ApplianceWizard
from ..modules import Qemu
from ..registry.appliance import Appliance, ApplianceError
from ..registry.registry import Registry
from ..registry.config import Config, ConfigException
from ..registry.image import Image
from ..utils import human_filesize
from ..utils.wait_for_lambda_worker import WaitForLambdaWorker
from ..utils.progress_dialog import ProgressDialog
from ..compute_manager import ComputeManager
from ..controller import Controller
from ..local_config import LocalConfig
from ..image_upload_manager import ImageUploadManager

import logging
log = logging.getLogger(__name__)


class ApplianceWizard(QtWidgets.QWizard, Ui_ApplianceWizard):
    images_changed_signal = QtCore.Signal()
    versions_changed_signal = QtCore.Signal()

    def __init__(self, parent, path):
        super().__init__(parent)

        self.setupUi(self)
        self.images_changed_signal.connect(self._refreshVersions)
        self.versions_changed_signal.connect(self._versionRefreshedSlot)

        self._refreshing = False

        self._path = path
        # Count how many images are curently uploading
        self._image_uploading_count = 0

        images_directories = list()
        images_directories.append(os.path.dirname(self._path))
        download_directory = QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.DownloadLocation)
        if download_directory != "" and download_directory != os.path.dirname(self._path):
            images_directories.append(download_directory)
        self._registry = Registry(images_directories)
        self._registry.image_list_changed_signal.connect(self.images_changed_signal.emit)

        self._appliance = Appliance(self._registry, self._path)

        self.uiApplianceVersionTreeWidget.currentItemChanged.connect(self._applianceVersionCurrentItemChangedSlot)
        self.uiRefreshPushButton.clicked.connect(self.images_changed_signal.emit)
        self.uiDownloadPushButton.clicked.connect(self._downloadPushButtonClickedSlot)
        self.uiImportPushButton.clicked.connect(self._importPushButtonClickedSlot)
        self.uiCreateVersionPushButton.clicked.connect(self._createVersionPushButtonClickedSlot)

        self.uiRemoteRadioButton.toggled.connect(self._remoteServerToggledSlot)
        if hasattr(self, "uiVMRadioButton"):
            self.uiVMRadioButton.toggled.connect(self._vmToggledSlot)

        self.uiLocalRadioButton.toggled.connect(self._localToggledSlot)
        if Controller.instance().isRemote():
            self.uiLocalRadioButton.setText("Install the appliance on the main server")
        else:
            if not path.endswith('.builtin.gns3a'):
                destination = Config().appliances_dir
                try:
                    os.makedirs(destination, exist_ok=True)
                    destination = os.path.join(destination, os.path.basename(path))
                    shutil.copy(path, destination)
                except OSError as e:
                    QtWidgets.QMessageBox.warning(self.parent(), "Can't copy {} to {}".format(path, destination), str(e))

        self.uiServerWizardPage.isComplete = self._uiServerWizardPage_isComplete

        # symbols loaded from controller
        self._symbols = []


    def initializePage(self, page_id):
        """
        Initialize Wizard pages.

        :param page_id: page identifier
        """
        super().initializePage(page_id)

        if self._appliance["category"] == "guest":
            symbol = ":/symbols/computer.svg"
        else:
            symbol = ":/symbols/{}.svg".format(self._appliance["category"])
        self.page(page_id).setPixmap(QtWidgets.QWizard.LogoPixmap, QtGui.QPixmap(symbol))

        if "qemu" in self._appliance:
            type = "qemu"
        elif "iou" in self._appliance:
            type = "iou"
        elif "docker" in self._appliance:
            type = "docker"
        elif "dynamips" in self._appliance:
            type = "dynamips"

        if self.page(page_id) == self.uiInfoWizardPage:
            Controller.instance().getSymbols(self._getSymbolsCallback)

            self.uiInfoWizardPage.setTitle(self._appliance["product_name"])
            self.uiDescriptionLabel.setText(self._appliance["description"])

            info = (
                ("Category", "category"),
                ("Product", "product_name"),
                ("Vendor", "vendor_name"),
                ("Status", "status"),
                ("Maintainer", "maintainer"),
                ("Architecture", "qemu/arch"),
                ("KVM", "qemu/kvm")
            )

            self.uiInfoTreeWidget.clear()
            for (name, key) in info:
                if "/" in key:
                    key, subkey = key.split("/")
                    value = self._appliance.get(key, {}).get(subkey, None)
                else:
                    value = self._appliance.get(key, None)
                if value is None:
                    continue
                item = QtWidgets.QTreeWidgetItem([name + ":", value])
                font = item.font(0)
                font.setBold(True)
                item.setFont(0, font)
                self.uiInfoTreeWidget.addTopLevelItem(item)

        elif self.page(page_id) == self.uiServerWizardPage:
            is_mac = ComputeManager.instance().localPlatform().startswith("darwin")
            is_win = ComputeManager.instance().localPlatform().startswith("win")

            self.uiRemoteServersComboBox.clear()
            if len(ComputeManager.instance().remoteComputes()) == 0:
                self.uiRemoteRadioButton.setEnabled(False)
            else:
                self.uiRemoteRadioButton.setEnabled(True)
                for compute in ComputeManager.instance().remoteComputes():
                    self.uiRemoteServersComboBox.addItem(compute.name(), compute)

            if not ComputeManager.instance().vmCompute():
                self.uiVMRadioButton.setEnabled(False)

            if ComputeManager.instance().localPlatform() is None:
                self.uiLocalRadioButton.setEnabled(False)
            elif is_mac or is_win:
                if type == "qemu":
                    # Qemu has issues on OSX and Windows we disallow usage of the local server
                    if not LocalConfig.instance().experimental():
                        self.uiLocalRadioButton.setEnabled(False)
                elif type != "dynamips":
                    self.uiLocalRadioButton.setEnabled(False)

            if ComputeManager.instance().vmCompute():
                self.uiVMRadioButton.setChecked(True)
            elif ComputeManager.instance().localCompute() and self.uiLocalRadioButton.isEnabled():
                self.uiLocalRadioButton.setChecked(True)
            elif self.uiRemoteRadioButton.isEnabled():
                self.uiRemoteRadioButton.setChecked(True)
            else:
                self.uiRemoteRadioButton.setChecked(False)

            if is_mac or is_win:
                if not self.uiRemoteRadioButton.isEnabled() \
                        and not self.uiVMRadioButton.isEnabled() \
                        and not self.uiLocalRadioButton.isEnabled():
                    QtWidgets.QMessageBox.warning(
                        self, "No GNS3 VM available.",
                        "GNS3 VM is not available, please configure GNS3 VM before adding new Appliance.")

        elif self.page(page_id) == self.uiFilesWizardPage:
            self._registry.getRemoteImageList(self._appliance.emulator(), self._compute_id)

        elif self.page(page_id) == self.uiQemuWizardPage:
            Qemu.instance().getQemuBinariesFromServer(self._compute_id, qpartial(self._getQemuBinariesFromServerCallback), [self._appliance["qemu"]["arch"]])

        elif self.page(page_id) == self.uiSummaryWizardPage:
            self.uiSummaryTreeWidget.clear()

            for key in self._appliance[type]:
                item = QtWidgets.QTreeWidgetItem([key.replace('_', ' ').capitalize() + ":", str(self._appliance[type][key])])
                font = item.font(0)
                font.setBold(True)
                item.setFont(0, font)
                self.uiSummaryTreeWidget.addTopLevelItem(item)
            self.uiSummaryTreeWidget.resizeColumnToContents(0)

        elif self.page(page_id) == self.uiUsageWizardPage:
            self.uiUsageTextEdit.setText("The appliance is available in the {} category. \n\n{}".format(
                self._appliance["category"].replace("_", " "),
                self._appliance.get("usage", ""))
            )

        elif self.page(page_id) == self.uiCheckServerWizardPage:
            self.uiCheckServerLabel.setText("Please wait while checking server capacities...")
            if 'qemu' in self._appliance:
                if self._appliance['qemu'].get('kvm', 'require') == 'require':
                    self._server_check = False  # If the server as the capacities for running the appliance
                    self.uiCheckServerLabel.setText("")
                    Qemu.instance().getQemuCapabilitiesFromServer(self._compute_id, qpartial(self._qemuServerCapabilitiesCallback))
                    return
            self.uiCheckServerLabel.setText("GNS3 server requirements is OK you can continue the installation")
            self._server_check = True

    def _qemuServerCapabilitiesCallback(self, result, error=None, *args, **kwargs):
        """
        Check if server support KVM or not
        """
        if error is None and "kvm" in result and self._appliance["qemu"]["arch"] in result["kvm"]:
            self._server_check = True
            self.uiCheckServerLabel.setText("GNS3 server requirements is OK you can continue the installation")
        else:
            if error:
                msg = result["message"]
            else:
                msg = "The remote server doesn't support KVM. You need a Linux server or the GNS3 VM with VMware and CPU virtualization instructions."
            self.uiCheckServerLabel.setText(msg)
            QtWidgets.QMessageBox.critical(self, "Qemu", msg)
            self._server_check = False

    def _uiServerWizardPage_isComplete(self):
        return self.uiRemoteRadioButton.isEnabled() or self.uiVMRadioButton.isEnabled() or self.uiLocalRadioButton.isEnabled()

    def _imageUploadedCallback(self, result, error=False, **kwargs):
        self._registry.getRemoteImageList(self._appliance.emulator(), self._compute_id)

    @qslot
    def _refreshVersions(self, *args):
        """
        Refresh the list of files for different version of the appliance
        """

        if self._refreshing:
            return
        self._refreshing = True

        self.uiFilesWizardPage.setSubTitle("The following versions are available for " + self._appliance["product_name"] + ".  Check the status of files required to install.")

        worker = WaitForLambdaWorker(lambda: self._refreshDialogWorker())
        progress_dialog = ProgressDialog(worker, "Add appliance", "Scanning directories for files...", None, busy=True, parent=self)
        progress_dialog.show()

    @qslot
    def _versionRefreshedSlot(self, *args):
        """
        Called when we finish to scan the disk for new versions
        """
        if self._refreshing or self.currentPage() != self.uiFilesWizardPage:
            return
        self._refreshing = True
        self.uiApplianceVersionTreeWidget.clear()

        for version in self._appliance["versions"]:
            top = QtWidgets.QTreeWidgetItem(self.uiApplianceVersionTreeWidget, ["{} {}".format(self._appliance["product_name"], version["name"])])
            size = 0
            status = "Ready to install"
            for image in version["images"].values():
                if image["status"] == "Missing":
                    status = "Missing files"

                size += image.get("filesize", 0)
                image_widget = QtWidgets.QTreeWidgetItem(
                    [
                        "",
                        image["filename"],
                        human_filesize(image.get("filesize", 0)),
                        image["status"],
                        image["version"],
                        image.get("md5sum", "")
                    ])
                if image["status"] == "Missing":
                    image_widget.setForeground(3, QtGui.QBrush(QtGui.QColor("red")))
                else:
                    image_widget.setForeground(3, QtGui.QBrush(QtGui.QColor("green")))

                # Associated data stored are col 0: version, col 1: image
                image_widget.setData(0, QtCore.Qt.UserRole, version)
                image_widget.setData(1, QtCore.Qt.UserRole, image)
                image_widget.setData(2, QtCore.Qt.UserRole, self._appliance)
                top.addChild(image_widget)

            font = top.font(0)
            font.setBold(True)
            top.setFont(0, font)

            expand = True
            if status == "Missing files":
                top.setForeground(3, QtGui.QBrush(QtGui.QColor("red")))
            else:
                expand = False
                top.setForeground(3, QtGui.QBrush(QtGui.QColor("green")))

            top.setData(2, QtCore.Qt.DisplayRole, human_filesize(size))
            top.setData(3, QtCore.Qt.DisplayRole, status)
            top.setData(2, QtCore.Qt.UserRole, self._appliance)
            top.setData(0, QtCore.Qt.UserRole, version)
            self.uiApplianceVersionTreeWidget.addTopLevelItem(top)
            # self.uiApplianceVersionTreeWidget.setCurrentItem(top)
            if expand:
                top.setExpanded(True)

        if len(self._appliance["versions"]) > 0:
            self.uiApplianceVersionTreeWidget.resizeColumnToContents(0)
            self.uiApplianceVersionTreeWidget.resizeColumnToContents(1)
        self._refreshing = False

    def _getSymbolsCallback(self, result, error=False, **kwargs):
        if error:
            log.warning("Cannot load symbols from controller")
        else:
            self._symbols = result

    def _refreshDialogWorker(self):
        """
        Scan local directory in order to found the images on disk
        """

        # Docker do not have versions
        if "versions" not in self._appliance:
            return

        for version in self._appliance["versions"]:
            for image in version["images"].values():
                img = self._registry.search_image_file(self._appliance.emulator(), image["filename"], image.get("md5sum"), image.get("filesize"))
                if img:
                    image["status"] = "Found"
                    image["md5sum"] = img.md5sum
                    image["filesize"] = img.filesize
                else:
                    image["status"] = "Missing"
        self._refreshing = False
        self.versions_changed_signal.emit()

    @qslot
    def _applianceVersionCurrentItemChangedSlot(self, current, previous):
        """
        Called when user select a different item in the list of appliance files
        """
        self.uiDownloadPushButton.hide()
        self.uiImportPushButton.hide()
        self.uiExplainDownloadLabel.hide()

        if current is None or sip.isdeleted(current):
            return

        image = current.data(1, QtCore.Qt.UserRole)
        if image is not None:
            if "direct_download_url" in image or "download_url" in image:
                self.uiDownloadPushButton.show()
            self.uiImportPushButton.show()

    @qslot
    def _downloadPushButtonClickedSlot(self, *args):
        """
        Called when user want to download an appliance images.
        He should have selected the file before.
        """
        if self._refreshing:
            return False

        current = self.uiApplianceVersionTreeWidget.currentItem()

        if current is None or sip.isdeleted(current):
            return

        data = current.data(1, QtCore.Qt.UserRole)
        if data is not None:
            if "direct_download_url" in data:
                QtGui.QDesktopServices.openUrl(QtCore.QUrl(data["direct_download_url"]))
                if "compression" in data:
                    QtWidgets.QMessageBox.warning(self, "Add appliance", "The file is compressed with {} you need to uncompress it before using it.".format(data["compression"]))
            else:
                QtWidgets.QMessageBox.warning(self, "Add appliance", "Download will redirect you where the required file can be downloaded, you may have to be registered with the vendor in order to download the file.")
                QtGui.QDesktopServices.openUrl(QtCore.QUrl(data["download_url"]))

    @qslot
    def _createVersionPushButtonClickedSlot(self, *args):
        """
        Allow user to create a new version of an appliance
        """

        new_version, ok = QtWidgets.QInputDialog.getText(self, "Creating a new version", "Creating a new version allows to import unknown files to use with this appliance.\nPlease share your experience on the GNS3 community if this version works.\n\nVersion name:", QtWidgets.QLineEdit.Normal)
        if ok:
            try:
                self._appliance.create_new_version(new_version)
            except ApplianceError as e:
                QtWidgets.QMessageBox.critical(self.parent(), "Create new version", str(e))
                return
            self.images_changed_signal.emit()

    @qslot
    def _importPushButtonClickedSlot(self, *args):
        """
        Called when user want to import an appliance images.
        He should have selected the file before.
        """
        if self._refreshing:
            return False

        current = self.uiApplianceVersionTreeWidget.currentItem()
        if not current:
            return
        disk = current.data(1, QtCore.Qt.UserRole)

        path, _ = QtWidgets.QFileDialog.getOpenFileName()
        if len(path) == 0:
            return

        image = Image(self._appliance.emulator(), path, filename=disk["filename"])
        try:
            if "md5sum" in disk and image.md5sum != disk["md5sum"]:
                QtWidgets.QMessageBox.warning(self.parent(), "Add appliance", "This is not the correct file. The MD5 sum is {} and should be {}.".format(image.md5sum, disk["md5sum"]))
                return
        except OSError as e:
            QtWidgets.QMessageBox.warning(self.parent(), "Add appliance", "Can't access to the image file {}: {}.".format(path, str(e)))
            return

        image_upload_manger = ImageUploadManager(
            image, Controller.instance(), self._compute_id,
            self._imageUploadedCallback, LocalConfig.instance().directFileUpload())
        image_upload_manger.upload()

    def _getQemuBinariesFromServerCallback(self, result, error=False, **kwargs):
        """
        Callback for getQemuBinariesFromServer.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            QtWidgets.QMessageBox.critical(self, "Qemu binaries", "{}".format(result["message"]))
        else:
            self.uiQemuListComboBox.clear()
            for qemu in result:
                if qemu["version"]:
                    self.uiQemuListComboBox.addItem("{path} (v{version})".format(path=qemu["path"], version=qemu["version"]), qemu["path"])
                else:
                    self.uiQemuListComboBox.addItem("{path}".format(path=qemu["path"]), qemu["path"])
            if self.uiQemuListComboBox.count() == 1:
                self.next()
            else:
                i = self.uiQemuListComboBox.findText(self._appliance["qemu"]["arch"], QtCore.Qt.MatchContains)
                if i != -1:
                    self.uiQemuListComboBox.setCurrentIndex(i)

    def _install(self, version):
        """
        Install the appliance to GNS3

        :params version: Version name
        """

        try:
            config = Config()
        except OSError as e:
            QtWidgets.QMessageBox.critical(self.parent(), "Add appliance", str(e))
            return False

        if version is None:
            appliance_configuration = self._appliance.copy()
        else:
            try:
                appliance_configuration = self._appliance.search_images_for_version(version)
            except ApplianceError as e:
                QtWidgets.QMessageBox.critical(self.parent(), "Add appliance", str(e))
                return False

        while len(appliance_configuration["name"]) == 0 or not config.is_name_available(appliance_configuration["name"]):
            QtWidgets.QMessageBox.warning(self.parent(), "Add appliance", "The name \"{}\" is already used by another appliance".format(appliance_configuration["name"]))
            appliance_configuration["name"], ok = QtWidgets.QInputDialog.getText(self.parent(), "Add appliance", "New name:", QtWidgets.QLineEdit.Normal, appliance_configuration["name"])
            if not ok:
                return False
            appliance_configuration["name"] = appliance_configuration["name"].strip()

        if "qemu" in appliance_configuration:
            appliance_configuration["qemu"]["path"] = self.uiQemuListComboBox.currentData()

        worker = WaitForLambdaWorker(
            lambda: config.add_appliance(appliance_configuration, self._compute_id, self._symbols),
            allowed_exceptions=[ConfigException, OSError])

        progress_dialog = ProgressDialog(worker, "Add appliance", "Install the appliance...", None, busy=True, parent=self)
        progress_dialog.show()
        if not progress_dialog.exec_():
            return False

        worker = WaitForLambdaWorker(lambda: config.save(), allowed_exceptions=[ConfigException, OSError])
        progress_dialog = ProgressDialog(worker, "Add appliance", "Install the appliance...", None, busy=True, parent=self)
        progress_dialog.show()
        if progress_dialog.exec_():
            QtWidgets.QMessageBox.information(self.parent(), "Add appliance", "{} installed!".format(appliance_configuration["name"]))
            return True

    def _uploadImages(self, version):
        """
        Upload an image to the compute
        """

        appliance_configuration = self._appliance.search_images_for_version(version)
        for image in appliance_configuration["images"]:
            if image["location"] == "local":
                image = Image(self._appliance.emulator(), image["path"], filename=image["filename"])

                image_upload_manger = ImageUploadManager(
                    image, Controller.instance(), self._compute_id,
                    self._applianceImageUploadedCallback, LocalConfig.instance().directFileUpload())
                image_upload_manger.upload()

                self._image_uploading_count += 1

    def _applianceImageUploadedCallback(self, result, error=False, **kwargs):
        self._image_uploading_count -= 1

    def nextId(self):
        if self.currentPage() == self.uiServerWizardPage:
            if "docker" in self._appliance:
                return super().nextId() + 3
            elif "qemu" not in self._appliance:
                return super().nextId() + 1
        elif self.currentPage() == self.uiFilesWizardPage:
            if "qemu" not in self._appliance:
                return super().nextId() + 1
        return super().nextId()

    def validateCurrentPage(self):
        """
        Validates the settings.
        """

        if self.currentPage() == self.uiFilesWizardPage:
            if self._refreshing:
                return False
            current = self.uiApplianceVersionTreeWidget.currentItem()
            if current is None or sip.isdeleted(current):
                return False
            version = current.data(0, QtCore.Qt.UserRole)
            if version is None:
                return False
            appliance = current.data(2, QtCore.Qt.UserRole)
            if not self._appliance.is_version_installable(version["name"]):
                QtWidgets.QMessageBox.warning(self, "Appliance", "Sorry, you cannot install {} with missing files".format(appliance["name"]))
                return False
            reply = QtWidgets.QMessageBox.question(self, "Appliance", "Would you like to install {} version {}?".format(appliance["name"], version["name"]),
                                                   QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
            if reply == QtWidgets.QMessageBox.No:
                return False
            self._uploadImages(version["name"])

        elif self.currentPage() == self.uiUsageWizardPage:
            if self._image_uploading_count > 0:
                QtWidgets.QMessageBox.critical(self, "Add appliance", "Please wait for image uploading")
                return False

            current = self.uiApplianceVersionTreeWidget.currentItem()
            if current:
                version = current.data(0, QtCore.Qt.UserRole)
                return self._install(version["name"])
            else:
                return self._install(None)

        elif self.currentPage() == self.uiServerWizardPage:
            if self.uiRemoteRadioButton.isChecked():
                if len(ComputeManager.instance().remoteComputes()) == 0:
                    QtWidgets.QMessageBox.critical(self, "Remote server", "There is no remote server registered in your preferences")
                    return False
                self._compute_id = self.uiRemoteServersComboBox.itemData(self.uiRemoteServersComboBox.currentIndex()).id()
            elif hasattr(self, "uiVMRadioButton") and self.uiVMRadioButton.isChecked():
                self._compute_id = "vm"
            else:
                if ComputeManager.instance().localPlatform():
                    if (ComputeManager.instance().localPlatform().startswith("darwin") or ComputeManager.instance().localPlatform().startswith("win")):
                        if "qemu" in self._appliance:
                            reply = QtWidgets.QMessageBox.question(self, "Appliance", "Qemu on Windows and MacOSX is not supported by the GNS3 team. Are you sur to continue?", QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
                            if reply == QtWidgets.QMessageBox.No:
                                return False

                self._compute_id = "local"

        elif self.currentPage() == self.uiQemuWizardPage:
            if self.uiQemuListComboBox.currentIndex() == -1:
                QtWidgets.QMessageBox.critical(self, "Qemu binary", "No compatible Qemu binary selected")
                return False

        elif self.currentPage() == self.uiCheckServerWizardPage:
            return self._server_check

        return True

    @qslot
    def _vmToggledSlot(self, checked):
        """
        Slot for when the VM radio button is toggled.

        :param checked: either the button is checked or not
        """
        if checked:
            self.uiRemoteServersGroupBox.setEnabled(False)
            self.uiRemoteServersGroupBox.hide()

    @qslot
    def _remoteServerToggledSlot(self, checked):
        """
        Slot for when the remote server radio button is toggled.

        :param checked: either the button is checked or not
        """

        if checked:
            self.uiRemoteServersGroupBox.setEnabled(True)
            self.uiRemoteServersGroupBox.show()

    @qslot
    def _localToggledSlot(self, checked):
        """
        Slot for when the local server radio button is toggled.

        :param checked: either the button is checked or not
        """
        if checked:
            self.uiRemoteServersGroupBox.setEnabled(False)
            self.uiRemoteServersGroupBox.hide()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Dialog to manage the snapshots.
"""

from ..qt import QtCore, QtWidgets
from ..ui.snapshots_dialog_ui import Ui_SnapshotsDialog
from ..controller import Controller
from ..utils.progress_dialog import ProgressDialog
from ..utils.create_snapshot_worker import CreateSnapshotWorker

from datetime import datetime

import logging
log = logging.getLogger(__name__)


class SnapshotsDialog(QtWidgets.QDialog, Ui_SnapshotsDialog):

    """
    Snapshots dialog implementation.

    :param parent: parent widget
    """

    def __init__(self, parent, project):

        super().__init__(parent)
        self.setupUi(self)

        self._project = project

        self.uiCreatePushButton.clicked.connect(self._createSnapshotSlot)
        self.uiDeletePushButton.clicked.connect(self._deleteSnapshotSlot)
        self.uiRestorePushButton.clicked.connect(self._restoreSnapshotSlot)
        self.uiSnapshotsList.itemDoubleClicked.connect(self._snapshotDoubleClickedSlot)
        self._listSnapshots()

    def _listSnapshots(self):
        """
        Lists all available snapshots.
        """

        self.uiSnapshotsList.clear()
        if self._project:
            Controller.instance().get("/projects/{}/snapshots".format(self._project.id()), self._listSnapshotsCallback)

    def _listSnapshotsCallback(self, result, error=False, server=None, context={}, **kwargs):
        if error:
            if result:
                log.error(result["message"])
            return

        for snapshot in result:
            item = QtWidgets.QListWidgetItem(self.uiSnapshotsList)
            item.setText("{} on {}".format(snapshot["name"], datetime.fromtimestamp(snapshot["created_at"]).strftime("%d/%m/%y at %H:%M:%S")))
            item.setData(QtCore.Qt.UserRole, snapshot["snapshot_id"])

        if self.uiSnapshotsList.count():
            self.uiSnapshotsList.setCurrentRow(0)
            self.uiDeletePushButton.setEnabled(True)
            self.uiRestorePushButton.setEnabled(True)
        else:
            self.uiDeletePushButton.setEnabled(False)
            self.uiRestorePushButton.setEnabled(False)

    def _createSnapshotSlot(self):
        """
        Slot to create a snapshot.
        """

        snapshot_name, ok = QtWidgets.QInputDialog.getText(self, "Snapshot", "Snapshot name:", QtWidgets.QLineEdit.Normal, "Unnamed")
        if ok and snapshot_name and self._project:
            snapshot_worker = CreateSnapshotWorker(self._project, snapshot_name)
            snapshot_worker.finished.connect(self._createSnapshotsCallback)

            progress_dialog = ProgressDialog(snapshot_worker, "Snapshot progress", "Creation of snapshot in progress...",
                                             "Cancel", busy=True, parent=self, create_thread=False, cancelable=True)
            progress_dialog.show()
            progress_dialog.exec_()


    def _createSnapshotsCallback(self):
        self._listSnapshots()

    def _createSnapshotsErrorCallback(self, message, error):
        log.error(message)

    def _deleteSnapshotSlot(self):
        """
        Slot to delete a snapshot.
        """

        item = self.uiSnapshotsList.currentItem()
        if item:
            snapshot_id = item.data(QtCore.Qt.UserRole)
            Controller.instance().delete("/projects/{}/snapshots/{}".format(self._project.id(), snapshot_id), self._deleteSnapshotsCallback)

    def _deleteSnapshotsCallback(self, result, error=False, server=None, context={}, **kwargs):
        if error:
            if result:
                log.error(result["message"])
            return
        self._listSnapshots()

    def _restoreSnapshotSlot(self):
        """
        Slot to restore a snapshot.
        """

        item = self.uiSnapshotsList.currentItem()
        if item:
            snapshot_id = item.data(QtCore.Qt.UserRole)
            self._restoreSnapshot(snapshot_id)

    def _restoreSnapshot(self, snapshot_id):
        """
        Restores a snapshot.

        :param snapshot_id: id of the snapshot
        """
        reply = QtWidgets.QMessageBox.question(self, "Snapshots", "This will discard any changes made to your project since the snapshot was taken?", QtWidgets.QMessageBox.Ok, QtWidgets.QMessageBox.Cancel)
        if reply == QtWidgets.QMessageBox.Cancel:
            return

        Controller.instance().post("/projects/{}/snapshots/{}/restore".format(self._project.id(), snapshot_id), self._restoreSnapshotsCallback, timeout=300)

    def _restoreSnapshotsCallback(self, result, error=False, server=None, context={}, **kwargs):
        if error:
            if result:
                log.error(result["message"])
            return
        self.accept()

    def _snapshotDoubleClickedSlot(self, item):
        """
        Slot to restore a snapshot when it is double clicked.
        """

        snapshot_id = item.data(QtCore.Qt.UserRole)
        self._restoreSnapshot(snapshot_id)



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import xml.etree.ElementTree as ET

from . import QtCore
from . import QtSvg
from . import QtGui

import logging
log = logging.getLogger(__name__)


class QImageSvgRenderer(QtSvg.QSvgRenderer):
    """
    Renderer pixmap and svg to SVG item

    :param path_or_data: Svg element of path to a SVG
    :param fallback: Image to display if the image is not working
    """

    def __init__(self, path_or_data=None, fallback=None):
        super().__init__()
        self._fallback = fallback
        self._svg = """<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="{width}" height="{height}"></svg>"""
        self.load(path_or_data)

    def load(self, path_or_data):
        try:
            path_exists = os.path.exists(path_or_data)
        except ValueError:  # On windows we can get an error because the path is too long (it's the svg data)
            path_exists = False

        if not path_exists and not path_or_data.startswith(":"):
            self._svg = path_or_data
            path_or_data = path_or_data.encode("utf-8")
            return super().load(path_or_data)

        try:
            # We load the SVG with ElementTree before
            # because Qt when failing loading send noise to logs
            # and their is no way to prevent that
            if not path_or_data.startswith(":") and path_exists:
                ET.parse(path_or_data)
            res = super().load(path_or_data)
            # If we can't render a SVG we load and base64 the image to create a SVG
            if self.isValid():
                return res
        except ET.ParseError:
            pass

        image = QtGui.QImage(path_or_data)
        data = QtCore.QByteArray()
        buf = QtCore.QBuffer(data)
        image.save(buf, 'PNG')
        if image.rect().width() > 0:
            self._svg = """<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="{width}" height="{height}">
    <image width="{width}" height="{height}" xlink:href="data:image/png;base64,{data}"/>
    </svg>""".format(data=bytes(data.toBase64()).decode(),
                     width=image.rect().width(),
                     height=image.rect().height())
            res = super().load(self._svg.encode())
        elif self._fallback:
            log.error("Invalid or corrupted image file")
            res = super().load(self._fallback)
        else:
            self._svg = """<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
                           </svg>"""
            res = super().load(self._svg.encode())
        return res

    def svg(self):
        """
        :returns: SVG source code
        """
        return self._svg



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Text editor to edit Note items.
"""

from ..qt import QtCore, QtWidgets, qslot
from ..ui.text_editor_dialog_ui import Ui_TextEditorDialog


class TextEditorDialog(QtWidgets.QDialog, Ui_TextEditorDialog):
    """
    Text editor dialog.

    :param parent: parent widget
    :param items: list of items
    """

    def __init__(self, parent, items):

        super().__init__(parent)
        self.setupUi(self)

        self._items = items
        self.uiFontPushButton.clicked.connect(self._setFontSlot)
        self.uiColorPushButton.clicked.connect(self._setColorSlot)
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Apply).clicked.connect(self._applyPreferencesSlot)

        # use the first item in the list as the model
        first_item = items[0]
        self._setColor(first_item.defaultTextColor())
        self.uiRotationSpinBox.setValue(first_item.rotation())
        self.uiPlainTextEdit.setPlainText(first_item.toPlainText())
        self.uiPlainTextEdit.setFont(first_item.font())

        if not first_item.editable():
            self.uiPlainTextEdit.setTextInteractionFlags(QtCore.Qt.NoTextInteraction)

        if len(self._items) == 1:
            self.uiApplyColorToAllItemsCheckBox.setChecked(True)
            self.uiApplyColorToAllItemsCheckBox.hide()
            self.uiApplyRotationToAllItemsCheckBox.setChecked(True)
            self.uiApplyRotationToAllItemsCheckBox.hide()
            self.uiApplyTextToAllItemsCheckBox.setChecked(True)
            self.uiApplyTextToAllItemsCheckBox.hide()

    def _setColor(self, color):
        self._color = color
        self.uiColorPushButton.setStyleSheet("background-color: rgba({}, {}, {}, {});".format(color.red(),
                                                                                              color.green(),
                                                                                              color.blue(),
                                                                                              color.alpha()))
        self.uiPlainTextEdit.setStyleSheet("color: rgba({}, {}, {}, {});".format(color.red(),
                                                                                 color.green(),
                                                                                 color.blue(),
                                                                                 color.alpha()))

    @qslot
    def _setFontSlot(self, *args):
        """
        Slot to select the font.
        """

        selected_font, ok = QtWidgets.QFontDialog.getFont(self.uiPlainTextEdit.font(), self,
                                                          options=QtWidgets.QFontDialog.DontUseNativeDialog)
        if ok:
            self.uiPlainTextEdit.setFont(selected_font)

    @qslot
    def _setColorSlot(self, *args):
        """
        Slot to select the color.
        """

        color = QtWidgets.QColorDialog.getColor(self._color, self, None, QtWidgets.QColorDialog.ShowAlphaChannel)
        if color.isValid():
            self._setColor(color)

    @qslot
    def _applyPreferencesSlot(self, *args):
        """
        Applies the new text settings.
        """

        for item in self._items:
            item.setFont(self.uiPlainTextEdit.font())
            if self.uiApplyColorToAllItemsCheckBox.isChecked():
                item.setDefaultTextColor(self._color)
            if self.uiApplyRotationToAllItemsCheckBox.isChecked():
                item.setRotation(self.uiRotationSpinBox.value())
            if item.editable() and self.uiApplyTextToAllItemsCheckBox.isChecked():
                item.setPlainText(self.uiPlainTextEdit.toPlainText())

    def done(self, result):
        """
        Called when the dialog is closed.

        :param result: boolean (accepted or rejected)
        """

        if result:
            self._applyPreferencesSlot()
        super().done(result)



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import xml.etree.ElementTree as ET

from gns3.items.ellipse_item import EllipseItem
from gns3.qt import QtGui, QtCore


def test_toSvg(project, controller):
    ellipse = EllipseItem(width=400, height=100, project=project)
    pen = QtGui.QPen(QtCore.Qt.black, 2, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
    pen.setStyle(QtCore.Qt.DashLine)
    ellipse.setPen(pen)
    svg = ET.fromstring(ellipse.toSvg())
    assert float(svg.get("width")) ==  400.0
    assert float(svg.get("height")) ==  100.0

    ellipse = svg[0]
    assert float(ellipse.get("cx")) ==  200.0
    assert float(ellipse.get("rx")) ==  200.0
    assert float(ellipse.get("cy")) ==  50.0
    assert float(ellipse.get("ry")) ==  50.0
    assert ellipse.get("stroke-width") == "2"
    assert ellipse.get("stroke") == "#000000"
    assert ellipse.get("fill", "#ffffff")

    assert ellipse.get("stroke-dasharray") == "25, 25"


def test_fromSvg(project, controller):
    ellipse = EllipseItem(project=project)
    ellipse.fromSvg('<svg height="150" width="250"><ellipse height="150" stroke-width="5" stroke="#0000ff" fill="#ff00ff" width="150" stroke-dasharray="5, 25, 25" /></svg>')
    assert ellipse.rect().width() == 250
    assert ellipse.rect().height() == 150
    assert ellipse.pen().width() == 5



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Graphical representation of an image on the QGraphicsScene.
"""

from ..qt import QtSvg
from ..qt.qimage_svg_renderer import QImageSvgRenderer
from .drawing_item import DrawingItem


class ImageItem(QtSvg.QGraphicsSvgItem, DrawingItem):

    """
    Class to insert an image on the scene.
    """

    def __init__(self, image_path=None, pos=None, svg=None, **kws):

        self._image_path = image_path
        # Because we call the Qt C++ code we need to handle the case of pos is None otherwise we will get a conversion error
        if pos:
            super().__init__(pos=pos, **kws)
        else:
            super().__init__(**kws)

        if self._image_path:
            renderer = QImageSvgRenderer(image_path)
            self.setSharedRenderer(renderer)

        # By default center the image
        if pos is None:
            x = self.pos().x() - (self.boundingRect().width() / 2)
            y = self.pos().y() - (self.boundingRect().height() / 2)
            self.setPos(x, y)

        if svg:
            svg = self.fromSvg(svg)

    def paint(self, painter, option, widget=None):
        """
        Paints the contents of an item in local coordinates.

        :param painter: QPainter instance
        :param option: QStyleOptionGraphicsItem instance
        :param widget: QWidget instance
        """

        super().paint(painter, option, widget)
        self.drawLayerInfo(painter)

    def setZValue(self, value):
        """
        Sets Z value of the item
        :param value: z layer
        """
        return DrawingItem.setZValue(self, value)

    def fromSvg(self, svg):
        renderer = QImageSvgRenderer(svg)
        self.setSharedRenderer(renderer)

    def toSvg(self):
        """
        Return an SVG version of the shape
        """
        return self.renderer().svg()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/traceng/ui/traceng_node_configuration_page.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_TraceNGNodeConfigPageWidget(object):
    def setupUi(self, TraceNGNodeConfigPageWidget):
        TraceNGNodeConfigPageWidget.setObjectName("TraceNGNodeConfigPageWidget")
        TraceNGNodeConfigPageWidget.resize(846, 340)
        self.gridLayout = QtWidgets.QGridLayout(TraceNGNodeConfigPageWidget)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(TraceNGNodeConfigPageWidget)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiIPAddressLabel = QtWidgets.QLabel(TraceNGNodeConfigPageWidget)
        self.uiIPAddressLabel.setObjectName("uiIPAddressLabel")
        self.gridLayout.addWidget(self.uiIPAddressLabel, 1, 0, 1, 1)
        self.uiDefaultDestinationLabel = QtWidgets.QLabel(TraceNGNodeConfigPageWidget)
        self.uiDefaultDestinationLabel.setObjectName("uiDefaultDestinationLabel")
        self.gridLayout.addWidget(self.uiDefaultDestinationLabel, 2, 0, 1, 2)
        self.uiDefaultNameFormatLabel = QtWidgets.QLabel(TraceNGNodeConfigPageWidget)
        self.uiDefaultNameFormatLabel.setObjectName("uiDefaultNameFormatLabel")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLabel, 3, 0, 1, 3)
        self.uiDefaultNameFormatLineEdit = QtWidgets.QLineEdit(TraceNGNodeConfigPageWidget)
        self.uiDefaultNameFormatLineEdit.setObjectName("uiDefaultNameFormatLineEdit")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLineEdit, 3, 3, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(TraceNGNodeConfigPageWidget)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout.addWidget(self.uiSymbolLabel, 4, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(TraceNGNodeConfigPageWidget)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.horizontalLayout_7.addWidget(self.uiSymbolLineEdit)
        self.uiSymbolToolButton = QtWidgets.QToolButton(TraceNGNodeConfigPageWidget)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.horizontalLayout_7.addWidget(self.uiSymbolToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_7, 4, 3, 1, 1)
        self.uiCategoryLabel = QtWidgets.QLabel(TraceNGNodeConfigPageWidget)
        self.uiCategoryLabel.setObjectName("uiCategoryLabel")
        self.gridLayout.addWidget(self.uiCategoryLabel, 5, 0, 1, 2)
        self.uiCategoryComboBox = QtWidgets.QComboBox(TraceNGNodeConfigPageWidget)
        self.uiCategoryComboBox.setObjectName("uiCategoryComboBox")
        self.gridLayout.addWidget(self.uiCategoryComboBox, 5, 3, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(263, 212, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 6, 1, 1, 3)
        self.uiDefaultDestinationLineEdit = QtWidgets.QLineEdit(TraceNGNodeConfigPageWidget)
        self.uiDefaultDestinationLineEdit.setObjectName("uiDefaultDestinationLineEdit")
        self.gridLayout.addWidget(self.uiDefaultDestinationLineEdit, 2, 3, 1, 1)
        self.uiIPAddressLineEdit = QtWidgets.QLineEdit(TraceNGNodeConfigPageWidget)
        self.uiIPAddressLineEdit.setObjectName("uiIPAddressLineEdit")
        self.gridLayout.addWidget(self.uiIPAddressLineEdit, 1, 3, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(TraceNGNodeConfigPageWidget)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 3, 1, 1)
        self.uiNameLabel.raise_()
        self.uiNameLineEdit.raise_()
        self.uiDefaultNameFormatLabel.raise_()
        self.uiDefaultNameFormatLineEdit.raise_()
        self.uiSymbolLabel.raise_()
        self.uiCategoryLabel.raise_()
        self.uiCategoryComboBox.raise_()
        self.uiIPAddressLabel.raise_()
        self.uiIPAddressLineEdit.raise_()
        self.uiDefaultDestinationLabel.raise_()
        self.uiDefaultDestinationLineEdit.raise_()

        self.retranslateUi(TraceNGNodeConfigPageWidget)
        QtCore.QMetaObject.connectSlotsByName(TraceNGNodeConfigPageWidget)

    def retranslateUi(self, TraceNGNodeConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        TraceNGNodeConfigPageWidget.setWindowTitle(_translate("TraceNGNodeConfigPageWidget", "TraceNG node configuration"))
        self.uiNameLabel.setText(_translate("TraceNGNodeConfigPageWidget", "Name:"))
        self.uiIPAddressLabel.setText(_translate("TraceNGNodeConfigPageWidget", "IP address:"))
        self.uiDefaultDestinationLabel.setText(_translate("TraceNGNodeConfigPageWidget", "Default destination:"))
        self.uiDefaultNameFormatLabel.setText(_translate("TraceNGNodeConfigPageWidget", "Default name format:"))
        self.uiSymbolLabel.setText(_translate("TraceNGNodeConfigPageWidget", "Symbol:"))
        self.uiSymbolToolButton.setText(_translate("TraceNGNodeConfigPageWidget", "&Browse..."))
        self.uiCategoryLabel.setText(_translate("TraceNGNodeConfigPageWidget", "Category:"))




#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from gns3.utils import parse_version


def test_parse_version():
    assert parse_version('1') == ('000001', '00000', '000000', 'final')
    assert parse_version('1.3') == ('000001', '000003', '000000', 'final')
    assert parse_version('1.3.dev3') == ('000001', '000003', '000000', 'dev', '000003')
    assert parse_version('1.3a1') == ('000001', '000003', '000000', 'a', '000001')
    assert parse_version('1.3rc1') == ('000001', '000003', '000000', 'c', '000001')

    assert parse_version('1.2.3') > parse_version('1.2.2')
    assert parse_version('1.3') > parse_version('1.2.2')
    assert parse_version('1.3') > parse_version('1.3alpha1')
    assert parse_version('1.3') > parse_version('1.3rc1')
    assert parse_version('1.3rc1') > parse_version('1.3alpha3')
    assert parse_version('1.3dev1') > parse_version('1.3rc1')
    assert parse_version('1.2.3') > parse_version('1.2')



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/edit_project_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_EditProjectDialog(object):
    def setupUi(self, EditProjectDialog):
        EditProjectDialog.setObjectName("EditProjectDialog")
        EditProjectDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        EditProjectDialog.resize(955, 387)
        EditProjectDialog.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(EditProjectDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.uiSceneWidthLabel = QtWidgets.QLabel(EditProjectDialog)
        self.uiSceneWidthLabel.setObjectName("uiSceneWidthLabel")
        self.gridLayout.addWidget(self.uiSceneWidthLabel, 1, 0, 1, 1)
        self.uiSceneHeightLabel = QtWidgets.QLabel(EditProjectDialog)
        self.uiSceneHeightLabel.setObjectName("uiSceneHeightLabel")
        self.gridLayout.addWidget(self.uiSceneHeightLabel, 2, 0, 1, 1)
        self.uiProjectAutoStartCheckBox = QtWidgets.QCheckBox(EditProjectDialog)
        self.uiProjectAutoStartCheckBox.setObjectName("uiProjectAutoStartCheckBox")
        self.gridLayout.addWidget(self.uiProjectAutoStartCheckBox, 7, 0, 1, 3)
        self.uiProjectAutoCloseCheckBox = QtWidgets.QCheckBox(EditProjectDialog)
        self.uiProjectAutoCloseCheckBox.setObjectName("uiProjectAutoCloseCheckBox")
        self.gridLayout.addWidget(self.uiProjectAutoCloseCheckBox, 8, 0, 1, 3)
        self.uiProjectNameLineEdit = QtWidgets.QLineEdit(EditProjectDialog)
        self.uiProjectNameLineEdit.setObjectName("uiProjectNameLineEdit")
        self.gridLayout.addWidget(self.uiProjectNameLineEdit, 0, 1, 1, 1)
        self.uiProjectNameLabel = QtWidgets.QLabel(EditProjectDialog)
        self.uiProjectNameLabel.setObjectName("uiProjectNameLabel")
        self.gridLayout.addWidget(self.uiProjectNameLabel, 0, 0, 1, 1)
        self.uiGridSizeLabel = QtWidgets.QLabel(EditProjectDialog)
        self.uiGridSizeLabel.setObjectName("uiGridSizeLabel")
        self.gridLayout.addWidget(self.uiGridSizeLabel, 3, 0, 1, 1)
        self.uiSceneWidthSpinBox = QtWidgets.QSpinBox(EditProjectDialog)
        self.uiSceneWidthSpinBox.setMinimum(500)
        self.uiSceneWidthSpinBox.setMaximum(1000000)
        self.uiSceneWidthSpinBox.setObjectName("uiSceneWidthSpinBox")
        self.gridLayout.addWidget(self.uiSceneWidthSpinBox, 1, 1, 1, 1)
        self.uiSceneHeightSpinBox = QtWidgets.QSpinBox(EditProjectDialog)
        self.uiSceneHeightSpinBox.setMinimum(500)
        self.uiSceneHeightSpinBox.setMaximum(1000000)
        self.uiSceneHeightSpinBox.setObjectName("uiSceneHeightSpinBox")
        self.gridLayout.addWidget(self.uiSceneHeightSpinBox, 2, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 10, 0, 1, 1)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(EditProjectDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.gridLayout.addWidget(self.uiButtonBox, 9, 0, 1, 3)
        self.uiProjectAutoOpenCheckBox = QtWidgets.QCheckBox(EditProjectDialog)
        self.uiProjectAutoOpenCheckBox.setObjectName("uiProjectAutoOpenCheckBox")
        self.gridLayout.addWidget(self.uiProjectAutoOpenCheckBox, 6, 0, 1, 3)
        self.uiGridSizeSpinBox = QtWidgets.QSpinBox(EditProjectDialog)
        self.uiGridSizeSpinBox.setObjectName("uiGridSizeSpinBox")
        self.gridLayout.addWidget(self.uiGridSizeSpinBox, 3, 1, 1, 1)

        self.retranslateUi(EditProjectDialog)
        self.uiButtonBox.accepted.connect(EditProjectDialog.accept)
        self.uiButtonBox.rejected.connect(EditProjectDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(EditProjectDialog)

    def retranslateUi(self, EditProjectDialog):
        _translate = QtCore.QCoreApplication.translate
        EditProjectDialog.setWindowTitle(_translate("EditProjectDialog", "Edit project"))
        self.uiSceneWidthLabel.setText(_translate("EditProjectDialog", "Scene width:"))
        self.uiSceneHeightLabel.setText(_translate("EditProjectDialog", "Scene height:"))
        self.uiProjectAutoStartCheckBox.setText(_translate("EditProjectDialog", "Start all nodes when this project is opened"))
        self.uiProjectAutoCloseCheckBox.setText(_translate("EditProjectDialog", "Leave this project running in the background when closing GNS3"))
        self.uiProjectNameLabel.setText(_translate("EditProjectDialog", "Project Name:"))
        self.uiGridSizeLabel.setText(_translate("EditProjectDialog", "Grid size:"))
        self.uiSceneWidthSpinBox.setSuffix(_translate("EditProjectDialog", " px"))
        self.uiSceneHeightSpinBox.setSuffix(_translate("EditProjectDialog", " px"))
        self.uiProjectAutoOpenCheckBox.setText(_translate("EditProjectDialog", "Open this project in the background when GNS3 server starts"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import uuid


from unittest.mock import patch, Mock, MagicMock
from gns3.modules.vpcs.vpcs_node import VPCSNode
from gns3.node import Node
from gns3.ports.port import Port
from gns3.ports.ethernet_port import EthernetPort
from gns3.ports.serial_port import SerialPort


def test_setupVMCallback(vpcs_device):
    node_id = str(uuid.uuid4())
    vpcs_device._createCallback = MagicMock()
    vpcs_device.createNodeCallback({
        "name": "PC 1",
        "node_id": node_id,
        "properties": {
            "startup_script": "echo TEST"
        }
    })
    assert vpcs_device._node_id == node_id
    assert vpcs_device._settings["startup_script"] == "echo TEST"
    vpcs_device._createCallback.assert_called_with({
        "name": "PC 1",
        "node_id": node_id,
        "startup_script": "echo TEST"
    }
    )


def test_vpcs_device_start(vpcs_device):

    with patch('gns3.base_node.BaseNode.controllerHttpPost') as mock:
        vpcs_device.start()
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/nodes/{node_id}/start".format(node_id=vpcs_device.node_id())


def test_vpcs_device_stop(vpcs_device):

    with patch('gns3.base_node.BaseNode.controllerHttpPost') as mock:
        vpcs_device.setStatus(Node.started)
        vpcs_device.stop()
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/nodes/{node_id}/stop".format(node_id=vpcs_device.node_id())


def test_vpcs_device_reload(vpcs_device):

    with patch('gns3.base_node.BaseNode.controllerHttpPost') as mock:
        vpcs_device.reload()
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/nodes/{node_id}/reload".format(node_id=vpcs_device.node_id())


def test_updatePorts(vpcs_device):
    vpcs_device._updatePorts([
        {
            "name": "Ethernet0/0",
            "short_name": "e0/0",
            "data_link_types": {"Ethernet": "DLT_EN10MB"},
            "port_number": 0,
            "adapter_number": 0,
            "link_type": "ethernet"
        }
    ])
    assert len(vpcs_device._ports) == 1
    port = vpcs_device._ports[0]
    assert port.name() == "Ethernet0/0"
    assert port.shortName() == "e0/0"
    assert port.portNumber() == 0
    assert port.adapterNumber() == 0
    assert port.dataLinkTypes() == {"Ethernet": "DLT_EN10MB"}
    assert port.status() == Port.stopped
    assert isinstance(port, EthernetPort)

    vpcs_device.setStatus(Node.started)
    vpcs_device._updatePorts([
        {
            "name": "Serial0/0",
            "short_name": "s0/0",
            "data_link_types": {},
            "port_number": 0,
            "adapter_number": 0,
            "link_type": "serial"
        }
    ])
    assert len(vpcs_device._ports) == 1
    port = vpcs_device._ports[0]
    assert port.status() == Port.started
    assert isinstance(port, SerialPort)


def test_updatePorts_PortChange(vpcs_device):
    """
    If the same port we do not recreate it but just update his informations
    """
    vpcs_device._updatePorts([
        {
            "name": "Ethernet0/0",
            "short_name": "e0/0",
            "data_link_types": {"Ethernet": "DLT_EN10MB"},
            "port_number": 0,
            "adapter_number": 0,
            "link_type": "ethernet"
        }
    ])
    port = vpcs_device._ports[0]

    vpcs_device.setStatus(Node.started)
    vpcs_device._updatePorts([
        {
            "name": "Ethernet0/0",
            "short_name": "e0/0",
            "data_link_types": {"Ethernet": "DLT_EN10MB"},
            "port_number": 0,
            "adapter_number": 0,
            "link_type": "ethernet"
        }
    ])
    assert port == vpcs_device._ports[0]
    assert port.status() == Port.started


def test_node_setGraphics(vpcs_device):
    node = MagicMock(
        pos=MagicMock(
            return_value=MagicMock(
                x=MagicMock(
                    return_value=10
                ),
                y=MagicMock(
                    return_value=20
                )
            )
        ),
        zValue=MagicMock(
            return_value=2
        ),
        symbol=MagicMock(
            return_value="symbol.svg"
        )
    )
    with patch('gns3.base_node.BaseNode.controllerHttpPut') as mock:
        vpcs_device.setGraphics(node)
        assert mock.call_count == 1
        args, kwargs = mock.call_args
        assert args[0] == "/nodes/{node_id}".format(node_id=vpcs_device.node_id())

        # second call should not make an update
        vpcs_device.setSettingValue('x', 10)
        vpcs_device.setSettingValue('y', 20)
        vpcs_device.setSettingValue('z', 2)
        vpcs_device.setSettingValue('symbol', "symbol.svg")
        vpcs_device.setSettingValue('label', node.label().dump())

        vpcs_device.setGraphics(node)
        assert mock.call_count == 1


# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gns3.qt import QtWidgets
from gns3.ui.capture_dialog_ui import Ui_CaptureDialog

import logging
log = logging.getLogger(__name__)


class CaptureDialog(QtWidgets.QDialog, Ui_CaptureDialog):
    """
    This dialog allow configure the packet capture
    """

    def __init__(self, parent, file_name, auto_start, ethernet_link=True):

        super().__init__(parent)
        self.setupUi(self)
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Ok).clicked.connect(self._okButtonClickedSlot)
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Cancel).clicked.connect(self.reject)

        if ethernet_link:
            self.uiDataLinkTypeComboBox.addItem("Ethernet", "DLT_EN10MB")
        else:
            self.uiDataLinkTypeComboBox.addItem("Cisco HDLC", "DLT_C_HDLC")
            self.uiDataLinkTypeComboBox.addItem("Cisco PPP", "DLT_PPP_SERIAL")
            self.uiDataLinkTypeComboBox.addItem("Frame Relay", "DLT_FRELAY")
            self.uiDataLinkTypeComboBox.addItem("ATM", "DLT_ATM_RFC1483")

        self.uiCaptureFileNameLineEdit.setText(file_name)
        self.uiStartCommandCheckBox.setChecked(auto_start)

    def _okButtonClickedSlot(self):
        if len(self.fileName()) == 0:
            QtWidgets.QMessageBox.warning(self.parent(), "Packet capture", "Please provide a file name for the capture")
            return
        self.accept()

    def fileName(self):
        return self.uiCaptureFileNameLineEdit.text()

    def dataLink(self):
        """
        Type of link for capture
        """
        return self.uiDataLinkTypeComboBox.currentData()

    def commandAutoStart(self):
        return self.uiStartCommandCheckBox.isChecked()


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    main = QtWidgets.QMainWindow()
    dialog = CaptureDialog(main, "test.pcap")
    dialog.show()
    exit_code = app.exec_()
    print(dialog.dataLink())
    print(dialog.fileName())



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import ssl
import http
import json
import base64
import urllib.request

import logging
log = logging.getLogger(__name__)


def getSynchronous(protocol, host, port, endpoint, timeout=2, user=None, password=None):
    """
    :returns: Tuple (Status code, json of anwser). Status 0 is a non HTTP error
    """
    try:
        url = "{protocol}://{host}:{port}/v2/{endpoint}".format(protocol=protocol, host=host, port=port, endpoint=endpoint)
        request = urllib.request.Request(url)

        if user is not None and len(user) > 0:
            log.debug("Synchronous get {} with user '{}'".format(url, user))
            base64string = base64.encodebytes('{}:{}'.format(user, password).encode()).replace(b'\n', b'')
            request.add_header("Authorization", "Basic {}".format(base64string.decode()))
        else:
            log.debug("Synchronous get {} (no authentication)".format(url))

        if sys.version_info >= (3, 5):
            ctx = ssl.create_default_context()
            ctx.check_hostname = False
            ctx.verify_mode = ssl.CERT_NONE
            response = urllib.request.urlopen(request, timeout=timeout, context=ctx)
        else:
            response = urllib.request.urlopen(request, timeout=timeout)

        content_type = response.getheader("CONTENT-TYPE")
        if response.status == 200:
            if content_type == "application/json":
                content = response.read()
                json_data = json.loads(content.decode("utf-8"))
                return response.status, json_data
        else:
            return response.status, None
    except http.client.InvalidURL as e:
        log.warn("Invalid local server url: {}".format(e))
        return 0, None
    except http.client.UnknownProtocol:
        log.warn("Unknown server running on {}:{}".format(host, port))
        return 0, None
    except urllib.error.URLError:
        # Connection refused. It's a normal behavior if server is not started
        return 0, None
    except urllib.error.HTTPError as e:
        log.debug("Error during get on {}:{}: {}".format(host, port, e))
        return e.code, None
    except (OSError, http.client.BadStatusLine, ValueError, SystemError) as e:
        log.debug("Error during get on {}:{}: {}".format(host, port, e))
    return 0, None



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .qt import QtCore
from .controller import Controller
from .utils.server_select import server_select

import logging
log = logging.getLogger(__name__)


class ApplianceManager(QtCore.QObject):

    appliances_changed_signal = QtCore.Signal()

    def __init__(self):
        super().__init__()
        self._appliance_templates = []
        self._appliances = []
        self._controller = Controller.instance()
        self._controller.connected_signal.connect(self.refresh)
        self._controller.disconnected_signal.connect(self._controllerDisconnectedSlot)
        self.refresh()

    def refresh(self):
        if self._controller.connected():
            self._controller.get("/appliances/templates", self._listApplianceTemplateCallback)
            self._controller.get("/appliances", self._listAppliancesCallback)

    def _controllerDisconnectedSlot(self):
        self._appliance_templates = []
        self._appliances = []
        self.appliances_changed_signal.emit()

    def appliance_templates(self):
        return self._appliance_templates

    def appliances(self):
        return self._appliances

    def getAppliance(self, appliance_id):
        """
        Look for an appliance by appliance ID
        """
        for appliance in self._appliances:
            if appliance["appliance_id"] == appliance_id:
                return appliance
        return None

    def _listAppliancesCallback(self, result, error=False, **kwargs):
        if error is True:
            log.error("Error while getting appliances list: {}".format(result["message"]))
            return
        self._appliances = result
        self.appliances_changed_signal.emit()

    def _listApplianceTemplateCallback(self, result, error=False, **kwargs):
        if error is True:
            log.error("Error while getting appliance templates list: {}".format(result["message"]))
            return
        self._appliance_templates = result
        self.appliances_changed_signal.emit()

    def createNodeFromApplianceId(self, project, appliance_id, x, y):
        for appliance in self._appliances:
            if appliance["appliance_id"] == appliance_id:
                break

        project_id = project.id()

        if appliance.get("compute_id") is None:
            from .main_window import MainWindow
            server = server_select(MainWindow.instance(), node_type=appliance["node_type"])
            if server is None:
                return False
            self._controller.post("/projects/" + project_id + "/appliances/" + appliance_id, self._createNodeFromApplianceCallback, {
                "compute_id": server.id(),
                "x": int(x),
                "y": int(y)
            },
                timeout=None)
        else:
            self._controller.post("/projects/" + project_id + "/appliances/" + appliance_id, self._createNodeFromApplianceCallback, {
                "x": int(x),
                "y": int(y)
            },
                timeout=None)
        return True

    def _createNodeFromApplianceCallback(self, result, error=False, **kwargs):
        if error:
            if "message" in result:
                log.error("Error while creating node: {}".format(result["message"]))
            return

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of ApplianceManager.
        :returns: instance of ApplianceManager
        """

        if not hasattr(ApplianceManager, '_instance') or ApplianceManager._instance is None:
            ApplianceManager._instance = ApplianceManager()
        return ApplianceManager._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Wizard for TraceNG nodes.
"""

import sys
import ipaddress

from gns3.qt import QtGui, QtWidgets
from gns3.node import Node
from gns3.dialogs.vm_wizard import VMWizard

from ..ui.traceng_node_wizard_ui import Ui_TraceNGNodeWizard


class TraceNGNodeWizard(VMWizard, Ui_TraceNGNodeWizard):

    """
    Wizard to create a TraceNG node template.

    :param parent: parent widget
    """

    def __init__(self, traceng_nodes, parent):

        super().__init__(traceng_nodes, parent)
        self.setPixmap(QtWidgets.QWizard.LogoPixmap, QtGui.QPixmap(":/icons/traceng.png"))
        self.uiNameWizardPage.registerField("name*", self.uiNameLineEdit)

        # TraceNG is only supported on a local server
        self.uiRemoteRadioButton.setEnabled(False)
        self.uiVMRadioButton.setEnabled(False)

    def validateCurrentPage(self):
        """
        Validates the server.
        """

        if super().validateCurrentPage() is False:
            return False

        if self.currentPage() == self.uiNameWizardPage:

            if not sys.platform.startswith("win"):
                QtWidgets.QMessageBox.critical(self, "TraceNG", "TraceNG can only run on Windows with a local server")
                return False

            ip_address = self.uiIPAddressLineEdit.text()
            if ip_address:
                try:
                    ipaddress.IPv4Address(ip_address)
                except ipaddress.AddressValueError:
                    QtWidgets.QMessageBox.critical(self, "IP address", "Invalid IP address format")
                    return False
        return True

    def getSettings(self):
        """
        Returns the settings set in this Wizard.

        :return: settings dict
        """

        settings = {"name": self.uiNameLineEdit.text(),
                    "ip_address": self.uiIPAddressLineEdit.text(),
                    "symbol": ":/symbols/traceng.svg",
                    "category": Node.end_devices,
                    "server": self._compute_id}

        return settings



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import copy

from gns3.qt import QtWidgets
from gns3.local_config import LocalConfig
from gns3.ui.console_command_dialog_ui import Ui_uiConsoleCommandDialog
from gns3.settings import PRECONFIGURED_TELNET_CONSOLE_COMMANDS, \
        PRECONFIGURED_VNC_CONSOLE_COMMANDS,                      \
        PRECONFIGURED_SPICE_CONSOLE_COMMANDS,                    \
        CUSTOM_CONSOLE_COMMANDS_SETTINGS


import logging
log = logging.getLogger(__name__)


class ConsoleCommandDialog(QtWidgets.QDialog, Ui_uiConsoleCommandDialog):
    """
    This dialog allow user to select the command used to start a
    console.
    """

    def __init__(self, parent, console_type="telnet", current=None):
        """
        :params console_type: telnet, serial, vnc or spice
        :params current: Current console command
        """
        super().__init__(parent)
        self.setupUi(self)
        self._console_type = console_type
        self._current = current

        self._settings = LocalConfig.instance().loadSectionSettings("CustomConsoleCommands", CUSTOM_CONSOLE_COMMANDS_SETTINGS)

        self.uiCommandComboBox.currentIndexChanged.connect(self.commandComboBoxCurrentIndexChangedSlot)
        self.uiCommandPlainTextEdit.textChanged.connect(self.textChangedSlot)
        self.uiSavePushButton.clicked.connect(self.savePushButtonClickedSlot)
        self.uiRemovePushButton.clicked.connect(self.removePushButtonClickedSlot)

        self._refreshList()

    def _refreshList(self):
        if self._console_type == "telnet":
            self._consoles = copy.copy(PRECONFIGURED_TELNET_CONSOLE_COMMANDS)
            self._consoles.update(self._settings[self._console_type])
        elif self._console_type == "vnc":
            self._consoles = copy.copy(PRECONFIGURED_VNC_CONSOLE_COMMANDS)
            self._consoles.update(self._settings[self._console_type])
        elif self._console_type == "spice":
            self._consoles = copy.copy(PRECONFIGURED_SPICE_CONSOLE_COMMANDS)
            self._consoles.update(self._settings[self._console_type])

        self.uiCommandComboBox.clear()
        self.uiCommandComboBox.addItem("Custom", "")
        for name, cmd in sorted(self._consoles.items(), key=(lambda item: item[0].lower())):
            self.uiCommandComboBox.addItem(name, cmd)

        if self._current:
            self.uiCommandPlainTextEdit.setPlainText(self._current)
        else:
            self.uiCommandComboBox.setCurrentIndex(1)

    def removePushButtonClickedSlot(self):
        """
        Remove the custom command from the custom list
        """
        self._settings[self._console_type].pop(self.uiCommandComboBox.currentText())
        LocalConfig.instance().saveSectionSettings("CustomConsoleCommands", self._settings)
        self._current = None
        self._refreshList()

    def savePushButtonClickedSlot(self):
        """
        Save a custom command to the list
        """
        name, ok = QtWidgets.QInputDialog.getText(self, "Add a command", "Command name:", QtWidgets.QLineEdit.Normal)
        command = self.uiCommandPlainTextEdit.toPlainText().strip()
        if ok and len(command) > 0:
            if command not in self._consoles.values():
                self._settings[self._console_type][name] = command
                self._current = command
                LocalConfig.instance().saveSectionSettings("CustomConsoleCommands", self._settings)
                self._refreshList()

    def textChangedSlot(self):
        index = self.uiCommandComboBox.findData(self.uiCommandPlainTextEdit.toPlainText())
        if index == -1:
            index = 0
        self.uiCommandComboBox.setCurrentIndex(index)

    def commandComboBoxCurrentIndexChangedSlot(self, index):
        self.uiRemovePushButton.hide()
        # Ignore custom command
        if index != 0:
            self.uiCommandPlainTextEdit.setPlainText(self.uiCommandComboBox.currentData())
            self.uiSavePushButton.hide()
            if self.uiCommandComboBox.currentText() in self._settings[self._console_type].keys():
                self.uiRemovePushButton.show()
        else:
            self.uiSavePushButton.show()

    @staticmethod
    def getCommand(parent, console_type="telnet", current=None):
        dialog = ConsoleCommandDialog(parent, console_type=console_type, current=current)
        dialog.show()
        if dialog.exec_():
            return (True, dialog.uiCommandPlainTextEdit.toPlainText().replace("\n", " "))
        return (False, None)


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    main = QtWidgets.QMainWindow()
    (ok, command) = ConsoleCommandDialog.getCommand(main, console_type="telnet", current=list(PRECONFIGURED_TELNET_CONSOLE_COMMANDS.items())[0][1])
    print(ok)
    print(command)




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..qt import QtWidgets
from ..topology import Topology
from ..ui.edit_project_dialog_ui import Ui_EditProjectDialog


class EditProjectDialog(QtWidgets.QDialog, Ui_EditProjectDialog):
    """
    Edit current project settings
    """

    def __init__(self, parent):

        super().__init__(parent)
        self.setupUi(self)
        self._project = Topology.instance().project()
        self.uiProjectNameLineEdit.setText(self._project.name())
        self.uiProjectAutoOpenCheckBox.setChecked(self._project.autoOpen())
        self.uiProjectAutoCloseCheckBox.setChecked(not self._project.autoClose())
        self.uiProjectAutoStartCheckBox.setChecked(self._project.autoStart())
        self.uiSceneWidthSpinBox.setValue(self._project.sceneWidth())
        self.uiSceneHeightSpinBox.setValue(self._project.sceneHeight())
        self.uiGridSizeSpinBox.setValue(self._project.gridSize())

    def done(self, result):
        """
        Called when the dialog is closed.

        :param result: boolean (accepted or rejected)
        """

        if result:
            self._project.setName(self.uiProjectNameLineEdit.text())
            self._project.setAutoOpen(self.uiProjectAutoOpenCheckBox.isChecked())
            self._project.setAutoClose(not self.uiProjectAutoCloseCheckBox.isChecked())
            self._project.setAutoStart(self.uiProjectAutoStartCheckBox.isChecked())
            self._project.setSceneHeight(self.uiSceneHeightSpinBox.value())
            self._project.setSceneWidth(self.uiSceneWidthSpinBox.value())
            self._project.setGridSize(self.uiGridSizeSpinBox.value())
            self._project.update()
        super().done(result)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/builtin/ui/ethernet_hub_configuration_page.ui'
#
# Created: Fri Jun 10 18:55:02 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ethernetHubConfigPageWidget(object):
    def setupUi(self, ethernetHubConfigPageWidget):
        ethernetHubConfigPageWidget.setObjectName("ethernetHubConfigPageWidget")
        ethernetHubConfigPageWidget.resize(507, 352)
        self.verticalLayout = QtWidgets.QVBoxLayout(ethernetHubConfigPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiSettingsGroupBox = QtWidgets.QGroupBox(ethernetHubConfigPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSettingsGroupBox.sizePolicy().hasHeightForWidth())
        self.uiSettingsGroupBox.setSizePolicy(sizePolicy)
        self.uiSettingsGroupBox.setObjectName("uiSettingsGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiSettingsGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiDefaultNameFormatLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiDefaultNameFormatLabel.setObjectName("uiDefaultNameFormatLabel")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLabel, 1, 0, 1, 2)
        self.uiDefaultNameFormatLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiDefaultNameFormatLineEdit.setObjectName("uiDefaultNameFormatLineEdit")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLineEdit, 1, 2, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout.addWidget(self.uiSymbolLabel, 2, 0, 1, 2)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.horizontalLayout_7.addWidget(self.uiSymbolLineEdit)
        self.uiSymbolToolButton = QtWidgets.QToolButton(self.uiSettingsGroupBox)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.horizontalLayout_7.addWidget(self.uiSymbolToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_7, 2, 2, 1, 1)
        self.uiCategoryLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiCategoryLabel.setObjectName("uiCategoryLabel")
        self.gridLayout.addWidget(self.uiCategoryLabel, 3, 0, 1, 2)
        self.uiCategoryComboBox = QtWidgets.QComboBox(self.uiSettingsGroupBox)
        self.uiCategoryComboBox.setObjectName("uiCategoryComboBox")
        self.gridLayout.addWidget(self.uiCategoryComboBox, 3, 2, 1, 1)
        self.uiPortsLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiPortsLabel.setObjectName("uiPortsLabel")
        self.gridLayout.addWidget(self.uiPortsLabel, 4, 0, 1, 1)
        self.uiPortsSpinBox = QtWidgets.QSpinBox(self.uiSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPortsSpinBox.sizePolicy().hasHeightForWidth())
        self.uiPortsSpinBox.setSizePolicy(sizePolicy)
        self.uiPortsSpinBox.setMinimum(0)
        self.uiPortsSpinBox.setMaximum(65535)
        self.uiPortsSpinBox.setProperty("value", 1)
        self.uiPortsSpinBox.setObjectName("uiPortsSpinBox")
        self.gridLayout.addWidget(self.uiPortsSpinBox, 4, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 71, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 5, 2, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 2, 1, 1)
        self.verticalLayout.addWidget(self.uiSettingsGroupBox)

        self.retranslateUi(ethernetHubConfigPageWidget)
        QtCore.QMetaObject.connectSlotsByName(ethernetHubConfigPageWidget)

    def retranslateUi(self, ethernetHubConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        ethernetHubConfigPageWidget.setWindowTitle(_translate("ethernetHubConfigPageWidget", "Ethernet hub"))
        self.uiSettingsGroupBox.setTitle(_translate("ethernetHubConfigPageWidget", "Settings"))
        self.uiNameLabel.setText(_translate("ethernetHubConfigPageWidget", "Name:"))
        self.uiDefaultNameFormatLabel.setText(_translate("ethernetHubConfigPageWidget", "Default name format:"))
        self.uiSymbolLabel.setText(_translate("ethernetHubConfigPageWidget", "Symbol:"))
        self.uiSymbolToolButton.setText(_translate("ethernetHubConfigPageWidget", "&Browse..."))
        self.uiCategoryLabel.setText(_translate("ethernetHubConfigPageWidget", "Category:"))
        self.uiPortsLabel.setText(_translate("ethernetHubConfigPageWidget", "Number of ports:"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import uuid
from unittest.mock import MagicMock

from gns3.topology import Topology


def test_topology_init():
    Topology()


def test_topology_node(vpcs_device):
    topology = Topology()
    topology.addNode(vpcs_device)
    assert len(topology.nodes()) == 1
    assert topology.getNode(vpcs_device.id()) == vpcs_device
    topology.removeNode(vpcs_device)
    assert len(topology.nodes()) == 0


def test_createDrawing_ellipse():
    topology = Topology()
    shape_data = {
        "x": 42,
        "y": 12,
        "z": 0,
        "rotation": 0,
        "drawing_id": str(uuid.uuid4()),
        "svg": "<svg height=\"105.0\" width=\"158.0\"><ellipse cx=\"79\" cy=\"52\" rx=\"79\" ry=\"53\" style=\"stroke-width:2;stroke:#000000;fill:#ffffff;\" /></svg>",
    }
    topology._main_window = MagicMock()
    topology.createDrawing(shape_data)
    topology._main_window.uiGraphicsView.createDrawingItem.assert_called_with("ellipse", 42, 12, 0, rotation=0, svg=shape_data["svg"], drawing_id=shape_data["drawing_id"])


def test_createDrawing_rect():
    topology = Topology()
    shape_data = {
        "x": 42,
        "y": 12,
        "z": 0,
        "rotation": 0,
        "drawing_id": str(uuid.uuid4()),
        "svg": "<svg height=\"105.0\" width=\"158.0\"><rect/></svg>",
    }
    topology._main_window = MagicMock()
    topology.createDrawing(shape_data)
    topology._main_window.uiGraphicsView.createDrawingItem.assert_called_with("rect", 42, 12, 0, rotation=0, svg=shape_data["svg"], drawing_id=shape_data["drawing_id"])


def test_createDrawing_text():
    topology = Topology()
    shape_data = {
        "x": 42,
        "y": 12,
        "z": 0,
        "rotation": 0,
        "drawing_id": str(uuid.uuid4()),
        "svg": "<svg height=\"105.0\" width=\"158.0\"><text/></svg>",
    }
    topology._main_window = MagicMock()
    topology.createDrawing(shape_data)
    topology._main_window.uiGraphicsView.createDrawingItem.assert_called_with("text", 42, 12, 0, rotation=0, svg=shape_data["svg"], drawing_id=shape_data["drawing_id"])


def test_createDrawing_svg():
    """
    If SVG is more complex we consider it as an image
    """
    topology = Topology()
    shape_data = {
        "x": 42,
        "y": 12,
        "z": 0,
        "rotation": 0,
        "drawing_id": str(uuid.uuid4()),
        "svg": "<svg height=\"105.0\" width=\"158.0\"><rect><line/></rect></svg>",
    }
    topology._main_window = MagicMock()
    topology.createDrawing(shape_data)
    topology._main_window.uiGraphicsView.createDrawingItem.assert_called_with("image", 42, 12, 0, rotation=0, svg=shape_data["svg"], drawing_id=shape_data["drawing_id"])

